#!/usr/bin/env python
# coding: utf-8
"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import sys
import os
from mock import MagicMock, patch, call

# Mock the winreg and _winreg module for non-windows python
_winreg = MagicMock()
sys.modules['_winreg']  = _winreg
sys.modules['winreg'] = _winreg

from mbed_lstools.windows import MbedLsToolsWin7

class Win7TestCase(unittest.TestCase):
    """ Basic test cases checking trivial asserts
    """

    def setUp(self):
        self.lstool = MbedLsToolsWin7()
        _winreg.HKEY_LOCAL_MACHINE = None
        _winreg.OpenKey.side_effect = None
        _winreg.EnumValue.side_effect = None
        _winreg.EnumKey.side_effect = None
        _winreg.QueryValue.side_effect = None
        _winreg.QueryInfoKey.side_effect = None
        _winreg.CreateKey.reset_mock()
        _winreg.CreateKeyEx.reset_mock()
        _winreg.DeleteKey.reset_mock()
        _winreg.DeleteKeyEx.reset_mock()
        _winreg.DeleteValue.reset_mock()
        _winreg.SetValue.reset_mock()
        _winreg.SetValueEx.reset_mock()
        _winreg.SaveKey.reset_mock()

    def test_os_supported(self):
        pass

    def test_empty_reg(self):
        value_dict = {
            (None, 'SYSTEM\\MountedDevices'): [
                ('\\DosDevices\\F:',
                 u'_??_USBSTOR#Disk&Ven_MBED&Prod_VFS&Rev_0.1#0240000032044e4500367009997b00086781000097969900&0#{53f56307-b6bf-11d0-94f2-00a0c91efb8b}'.encode('utf-16le')),
                ('\\DosDevices\\D:',
                 u'_??_USBSTOR#Disk&Ven_SEGGER&Prod_MSD_Volume&Rev_1.00#8&1b8e102b&0&000440035522&0#{53f56307-b6bf-11d0-94f2-00a0c91efb8b}'.encode('utf-16le'))],
            ((None, 'SYSTEM\\CurrentControlSet'), 'Services\\usbccgp\\Enum'): [
                ('Count', 0),
                ('NextInstance', 0)
            ]
        }
        key_dict = {
            (None, 'SYSTEM\\CurrentControlSet'): ['Services\\usbccgp\\Enum'],
            (None, 'SYSTEM\\CurrentControlSet'): ['Enum\\USB'],
            ((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'):
            ['ROOT_HUB30', 'VID_0416&PID_511E', 'VID_0416&PID_511E&MI_00',
             'VID_8087&PID_0A2B', 'Vid_80EE&Pid_CAFE']
        }
        self.setUpRegistry(value_dict, key_dict)
        candidates = self.lstool.find_candidates()
        self.assertEqual(_winreg.OpenKey.mock_calls, [
            call(_winreg.HKEY_LOCAL_MACHINE, 'SYSTEM\\CurrentControlSet'),
            call((_winreg.HKEY_LOCAL_MACHINE, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
            call((_winreg.HKEY_LOCAL_MACHINE, 'SYSTEM\\CurrentControlSet'), 'Services\\usbccgp\\Enum'),
            call((_winreg.HKEY_LOCAL_MACHINE, 'SYSTEM\\CurrentControlSet'), 'Services\\mbedComposite\\Enum')
        ])
        self.assertEqual(candidates, [])

    def assertNoRegMut(self):
        """Assert that the registry was not mutated in this test"""
        _winreg.CreateKey.assert_not_called()
        _winreg.CreateKeyEx.assert_not_called()
        _winreg.DeleteKey.assert_not_called()
        _winreg.DeleteKeyEx.assert_not_called()
        _winreg.DeleteValue.assert_not_called()
        _winreg.SetValue.assert_not_called()
        _winreg.SetValueEx.assert_not_called()
        _winreg.SaveKey.assert_not_called()

    def setUpRegistry(self, value_dict, key_dict):
        all_keys = set(value_dict.keys()) | set(key_dict.keys())
        def open_key_effect(key, subkey):
            if ((key, subkey) in all_keys or key in all_keys):
                return key, subkey
            else:
                raise OSError((key, subkey))
        _winreg.OpenKey.side_effect = open_key_effect
        def enum_value(key, index):
            try:
                a, b = value_dict[key][index]
                return a, b, None
            except KeyError:
                raise OSError
        _winreg.EnumValue.side_effect = enum_value
        def enum_key(key, index):
            try:
                return key_dict[key][index]
            except KeyError:
                raise OSError
        _winreg.EnumKey.side_effect = enum_key
        def query_value(key, subkey):
            try:
                return value_dict[(key, subkey)]
            except KeyError:
                raise OSError
        _winreg.QueryValueEx.side_effect = query_value
        def query_info_key(key):
            return (len(key_dict.get(key, [])),
                    len(value_dict.get(key, [])))
        _winreg.QueryInfoKey.side_effect = query_info_key

    def test_one_dev(self):
        value_dict = {
            (None, 'SYSTEM\\MountedDevices'): [
                ('\\DosDevices\\C:', u'NOT A VALID MBED DRIVE'.encode('utf-16le')),
                ('\\DosDevices\\F:',
                 u'_??_USBSTOR#Disk&Ven_MBED&Prod_VFS&Rev_0.1#11010000442031204c364141303031313431303397969903&0#{53f56307-b6bf-11d0-94f2-00a0c91efb8b}'.encode('utf-16le'))
            ],
            ((None, 'SYSTEM\\CurrentControlSet'), 'Services\\usbccgp\\Enum'): [
                ('0', 'USB\\VID_5986&PID_0706\\5&31ac2c0b&0&8'),
                ('Count', 2),
                ('NextInstance', 2)
            ],
            ((None, 'SYSTEM\\CurrentControlSet'), 'Services\\mbedComposite\\Enum'): [
                ('0', 'USB\\VID_0D28&PID_0204\\11010000442031204c364141303031313431303397969903'),
                ('Count', 1),
                ('NextInstance', 1)
            ],
            ((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'), 'VID_5986&PID_0706\\5&31ac2c0b&0&8'), 'ParentIdPrefix'): ('6&3beba67&0', None),
            ((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'), 'VID_5986&PID_0706&MI_00\\6&3beba67&0&0000'),
                'Service'): ('SPUVCbv', None),
            (((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204\\11010000442031204c364141303031313431303397969903'): [],
            ((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_00\\11010000442031204c364141303031313431303397969903'),
                'Service'): ('USBSTOR', None),
            ((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_01\\11010000442031204c364141303031313431303397969903'),
                'Device Parameters'): [('PortName', 'COM7')],
            (((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_01\\11010000442031204c364141303031313431303397969903'),
                'Device Parameters'), 'PortName'): ('COM7', None),
            ((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_03\\11010000442031204c364141303031313431303397969903'),
                'Service'): ('HidUsb', None)
        }
        key_dict = {
            (None, 'SYSTEM\\CurrentControlSet'): ['Services\\usbccgp\\Enum', 'Services\\mbedComposite\\Enum'],
            (None, 'SYSTEM\\CurrentControlSet'): ['Enum\\USB'],
            ((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'):
            ['ROOT_HUB30', 'VID_0416&PID_511E', 'VID_0416&PID_511E&MI_00',
             'VID_0416&PID_511E&MI_01', 'VID_046D&PID_C03D',
             'VID_046D&PID_C313', 'VID_046D&PID_C313&MI_00',
             'VID_046D&PID_C313&MI_01', 'VID_046D&PID_C52B',
             'VID_046D&PID_C52B&MI_00', 'VID_046D&PID_C52B&MI_01',
             'VID_046D&PID_C52B&MI_02', 'VID_0483&PID_374B',
             'VID_0483&PID_374B&MI_00', 'VID_0483&PID_374B&MI_01',
             'VID_0483&PID_374B&MI_02', 'VID_0930&PID_6545',
             'VID_0D28&PID_0204', 'VID_0D28&PID_0204&MI_00',
             'VID_0D28&PID_0204&MI_01', 'VID_0D28&PID_0204&MI_03',
             'VID_1366&PID_1015', 'VID_1366&PID_1015&MI_00',
             'VID_1366&PID_1015&MI_02', 'VID_1366&PID_1015&MI_03',
             'VID_138A&PID_0090', 'VID_17EF&PID_100F', 'VID_17EF&PID_1010',
             'VID_17EF&PID_6019', 'VID_18D1&PID_4EE1', 'VID_195D&PID_2047',
             'VID_195D&PID_2047&MI_00', 'VID_195D&PID_2047&MI_01',
             'VID_195D&PID_2047&MI_02', 'VID_1A40&PID_0101', 'VID_1FD2&PID_5003',
             'VID_1FD2&PID_5003&MI_00', 'VID_1FD2&PID_5003&MI_01',
             'VID_413C&PID_2107', 'VID_5986&PID_0706', 'VID_5986&PID_0706&MI_00',
             'VID_8087&PID_0A2B', 'Vid_80EE&Pid_CAFE'],
            (((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_5986&PID_0706&MI_00'): ['6&3beba67&0&0000'],
            (((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_00'): ['11010000442031204c364141303031313431303397969903'],
            (((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_01'): ['11010000442031204c364141303031313431303397969903'],
            ((((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_01'), '11010000442031204c364141303031313431303397969903'): ['Device Parameters'],
            (((None, 'SYSTEM\\CurrentControlSet'), 'Enum\\USB'),
                'VID_0D28&PID_0204&MI_03'): ['11010000442031204c364141303031313431303397969903']
        }
        self.setUpRegistry(value_dict, key_dict)

        with patch('mbed_lstools.windows.MbedLsToolsWin7._run_cli_process') as _cliproc:
            _cliproc.return_value = ("", "", 0)
            expected_info = {
                'mount_point': u'F:',
                'serial_port': 'COM7',
                'target_id_usb_id': u'11010000442031204c364141303031313431303397969903'
            }

            devices = self.lstool.find_candidates()
            self.assertIn(expected_info, devices)
            self.assertNoRegMut()

    def test_mount_point_ready(self):
        with patch('mbed_lstools.windows.MbedLsToolsWin7._run_cli_process') as _cliproc:
            _cliproc.return_value = ("dummy", "", 0)
            self.assertTrue(self.lstool.mount_point_ready("dummy"))

            _cliproc.reset_mock()

            _cliproc.return_value = ("", "dummy", 1)
            self.assertFalse(self.lstool.mount_point_ready("dummy"))


if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
import os
import sys
import string
import itertools

from .lstools_base import MbedLsToolsBase

import logging
logger = logging.getLogger("mbedls.lstools_win7")
logger.addHandler(logging.NullHandler())
DEBUG = logging.DEBUG
del logging

if sys.version_info[0] < 3:
    import _winreg as winreg
else:
    import winreg

def _composite_device_key(device):
    """! Given two composite devices, return a ranking on its specificity
    @return if no mount_point, then always 0. If mount_point but no serial_port,
    return 1. If mount_point and serial_port, add the prefix_index.
    """
    rank = 0

    if 'mount_point' in device:
        rank += 1
        if device['serial_port'] is not None:
            rank += device['prefix_index']

    return rank


class MbedLsToolsWin7(MbedLsToolsBase):
    """ mbed-enabled platform detection for Windows
    """

    _COMPOSITE_USB_SERVICES = ['usbccgp', 'mbedComposite']

    def __init__(self, **kwargs):
        MbedLsToolsBase.__init__(self, **kwargs)
        self.os_supported.append('Windows7')


    def find_candidates(self):
        result = []
        composite_devices = self._get_composite_devices()
        if composite_devices:
            volumes = self._get_volumes()
            for composite_device in composite_devices:
                tid = composite_device['target_id_usb_id']
                found_index = None
                for index, volume in enumerate(volumes):
                    if volume['target_id_usb_id'] == tid:
                        found_index = index
                        composite_device.update(volume)
                        result.append(composite_device)

                if found_index is not None:
                    volumes.pop(found_index)
        return result


    def _get_volumes(self):
        """! Get the volumes present on the system
        @return List of mount points and their associated target id
          Ex. [{ 'mount_point': 'D:', 'target_id_usb_id': 'xxxx'}, ...]
        """
        result = []
        try:
            # Open the registry key for mounted devices
            mounted_devices_key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                'SYSTEM\\MountedDevices')
            for v in self.iter_vals(mounted_devices_key):
                # Valid entries have the following format: \DosDevices\D:
                if 'DosDevices' in v[0]:
                    entry_string = v[1].decode('utf-16le', 'ignore')
                    mount_point_match = re.match('.*\\\\(.:)$', v[0])

                    if not mount_point_match:
                        logger.debug('Invalid disk pattern for entry %s, skipping' % v[0])
                        continue

                    mount_point = mount_point_match.group(1)
                    logger.debug('Mount point %s found for volume %s' % (mount_point,
                        entry_string))

                    # TargetID is a hex string with 10-48 chars
                    target_id_match = re.search('[&#]([0-9A-Za-z]{10,48})[&#]', entry_string)
                    if not target_id_match:
                        logger.debug('Entry %s has invalid target id pattern '
                            '%s, skipping' % (v[0], entry_string))
                        continue

                    target_id = target_id_match.group(1)
                    logger.debug('Target ID %s found for volume %s' % (target_id,
                        entry_string))

                    result.append({
                        'mount_point': mount_point,
                        'target_id_usb_id': target_id
                    })
        except OSError:
            logger.error('Failed to open "MountedDevices" in registry')

        return result


    def _get_composite_devices(self):
        """! Get connected composite USB devices
        @return List of target ids and serial properties
          Ex. [{ 'target_id_usb_id': 'xxxx', 'serial_port': 'COMxx', 'mount_point': None }, ...]
        @details The composite devices are required to have a mass storage device.
            If a mass storage is present, a key of 'mount_point' should be returned
            with a value of None (to be completed later)
        """

        result = []

        # Open the root registry key
        try:
            control_set_key_string = "SYSTEM\\CurrentControlSet"
            control_set_key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                control_set_key_string)
        except OSError:
            logger.error('Could not find "%s" in registry' % control_set_key_string)
            return []

        # Enumerate all USB VID/PID pairs
        try:
            vid_pid_key = winreg.OpenKey(control_set_key, 'Enum\\USB')
            vid_pid_keys = list(self.iter_keys_as_str(vid_pid_key))
        except OSError:
            logger.error('Could not enumerate USB VID/PID pairs')
            return []

        # Find all composite USB services
        found_composite_keys = []
        for usb_service in self._COMPOSITE_USB_SERVICES:
            try:
                found_composite_keys.append(winreg.OpenKey(control_set_key,
                    'Services\\%s\\Enum' % usb_service))
            except OSError:
                logger.debug('Composite USB service "%s" not found' % usb_service)

        # Find all enumerated composite USB devices
        composite_iter_vals = [self.iter_vals(k) for k in found_composite_keys]
        for point, label, _ in itertools.chain.from_iterable(composite_iter_vals):
            try:
                # These keys in the registry enumerate all composite DosDevices
                # as a value with an integer. This check ensures we ignore a few
                # helper values in the registry key.
                _ = int(point)
            except ValueError:
                continue

            label_parts = label.split('\\')

            # The expected format is "USB\VID_XXXX&PID_XXXX\<target id>"
            if len(label_parts) != 3:
                logger.debug('Unrecognized device path %s' % label)
                continue

            if label_parts[0] != 'USB':
                logger.debug('Expected device to be under "USB", '
                    'actual location was "%s"' % label_parts[0])
                continue

            vid_pid_prefix = label_parts[1]
            target_id_usb_id = label_parts[2]

            device = {}

            try:
                composite_key_string = '%s\\%s' % (vid_pid_prefix, target_id_usb_id)
                logger.debug('Composite device at key %s' % composite_key_string)
                composite_key = winreg.OpenKey(vid_pid_key, composite_key_string)
                # Order is important here. We should default to using the
                # target_id_usb_id here but fallback to the ParentIdPrefix
                parent_id_prefixes = [target_id_usb_id]
                try:
                    composite_parent_id_prefix, _ = winreg.QueryValueEx(composite_key,
                        'ParentIdPrefix')
                    parent_id_prefixes.append(composite_parent_id_prefix)
                except OSError:
                    logger.debug('No ParentIdPrefix found')

                vid_pid_matches = [
                    m for m in vid_pid_keys
                    if m.startswith(vid_pid_prefix) and m != vid_pid_prefix
                ]

                options = []

                for prefix_index, parent_id_prefix in enumerate(parent_id_prefixes):
                    device = {
                        'target_id_usb_id': target_id_usb_id,
                        'serial_port': None,
                        'prefix_index': len(parent_id_prefixes) - prefix_index
                    }

                    for vid_pid_match in vid_pid_matches:
                        logger.debug('Endpoint parent at key %s' % vid_pid_match)
                        endpoint_parent_key = winreg.OpenKey(vid_pid_key, vid_pid_match)

                        candidates = [
                            c for c in list(self.iter_keys_as_str(endpoint_parent_key))
                            if any(c.startswith(parent_id_prefix) for p in parent_id_prefixes)
                        ]

                        if len(candidates) == 0:
                            logger.debug('No candidate found for parent_id_prefix'
                                ' %s' % parent_id_prefix)
                            break
                        elif len(candidates) > 1:
                            logger.debug('Unexpectedly found two candidates %s' % candidates)
                            logger.debug('Picking %s and continuing' % candidates[0])

                        endpoint_key_string = '%s\\%s' % (vid_pid_match, candidates[0])
                        logger.debug('Endpoint at key %s' % endpoint_key_string)
                        endpoint_key = winreg.OpenKey(vid_pid_key, endpoint_key_string)
                        # This verifies that a USB Storage device is associated with
                        # the composite device
                        if not 'mount_point' in device:
                            try:
                                service, _ = winreg.QueryValueEx(endpoint_key, 'Service')
                                if service.upper() == 'USBSTOR':
                                    device['mount_point'] = None
                                    continue
                            except OSError:
                                pass

                        # This adds the serial port information for the device if
                        # it is availble
                        if device['serial_port'] is None:
                            try:
                                device_parameters_key = winreg.OpenKey(endpoint_key,
                                    'Device Parameters')
                                device['serial_port'], _ = winreg.QueryValueEx(
                                    device_parameters_key, 'PortName')
                                continue
                            except OSError:
                                pass

                    options.append(device)

            except OSError as e:
                logger.debug('Skipping device entry %s, %s' % (label, e))
                continue

            options.sort(key=_composite_device_key, reverse=True)

            if len(options) == 0:
                logger.debug('No options were found, skipping composite device')
                continue


            device = options[0]
            del device['prefix_index']

            # A target_id_usb_id and mount point must be present to be included
            # in the candidates
            required_keys = set(['target_id_usb_id', 'mount_point'])
            missing_keys = required_keys - set(device.keys())
            if missing_keys:
                logger.debug('Device %s missing keys %s, skipping' % (label,
                    list(missing_keys)))
            else:
                logger.debug('Candidate found %s' % label)
                result.append(device)

        return result


    # =============================== Registry ====================================

    def iter_keys_as_str(self, key):
        """! Iterate over subkeys of a key returning subkey as string
        """
        for i in range(winreg.QueryInfoKey(key)[0]):
            yield winreg.EnumKey(key, i)


    def iter_keys(self, key):
        """! Iterate over subkeys of a key
        """
        for i in range(winreg.QueryInfoKey(key)[0]):
            yield winreg.OpenKey(key, winreg.EnumKey(key, i))


    def iter_vals(self, key):
        """! Iterate over values of a key
        """
        logger.debug("iter_vals %r", key)
        for i in range(winreg.QueryInfoKey(key)[1]):
            yield winreg.EnumValue(key, i)

    def mount_point_ready(self, path):
        """! Check if a mount point is ready for file operations
        @return Returns True if the given path exists, False otherwise
        @details Calling the Windows command `dir` instead of using the python
        `os.path.exists`. The latter causes a Python error box to appear claiming
        there is "No Disk" for some devices that are in the ejected state. Calling
        `dir` prevents this since it uses the Windows API to determine if the
        device is ready before accessing the file system.
        """
        stdout, stderr, retcode = self._run_cli_process('dir %s' % path)
        result = True if retcode == 0 else False

        if result:
            logger.debug("Mount point %s is ready", path)
        else:
            logger.debug("Mount point %s reported not ready with error '%s'",
                          path, stderr.strip())

        return result



#!/usr/bin/env python
'''
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

import unittest
import sys
import os
from mock import patch
from mbed_lstools.linux import MbedLsToolsLinuxGeneric

class LinuxPortTestCase(unittest.TestCase):
    ''' Basic test cases checking trivial asserts
    '''

    def setUp(self):
        self.linux_generic = MbedLsToolsLinuxGeneric()

    def tearDown(self):
        pass

    vfat_devices = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/usb2 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/usb3 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]

    def test_get_mount_point_basic(self):
        with patch('mbed_lstools.linux.MbedLsToolsLinuxGeneric._run_cli_process') as _cliproc:
            _cliproc.return_value = (b'\n'.join(self.vfat_devices), None, 0)
            mount_dict = dict(self.linux_generic._fat_mounts())
            _cliproc.assert_called_once_with('mount')
        self.assertEqual('/media/usb0', mount_dict['/dev/sdb'])
        self.assertEqual('/media/usb2', mount_dict['/dev/sdd'])
        self.assertEqual('/media/usb3', mount_dict['/dev/sde'])
        self.assertEqual('/media/usb1', mount_dict['/dev/sdc'])


    vfat_devices_ext = [
        b'/dev/sdb on /media/MBED_xxx type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/MBED___x type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/MBED-xxx type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/MBED_x-x type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',

        b'/dev/sda on /mnt/NUCLEO type vfat (rw,relatime,uid=999,fmask=0133,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,utf8,flush,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdf on /mnt/NUCLEO_ type vfat (rw,relatime,uid=999,fmask=0133,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,utf8,flush,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdg on /mnt/DAPLINK type vfat (rw,relatime,sync,uid=999,fmask=0022,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdh on /mnt/DAPLINK_ type vfat (rw,relatime,sync,uid=999,fmask=0022,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdi on /mnt/DAPLINK__ type vfat (rw,relatime,sync,uid=999,fmask=0022,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,errors=remount-ro,uhelper=ldm)',
    ]

    def test_get_mount_point_ext(self):
        with patch('mbed_lstools.linux.MbedLsToolsLinuxGeneric._run_cli_process') as _cliproc:
            _cliproc.return_value = (b'\n'.join(self.vfat_devices_ext), None, 0)
            mount_dict = dict(self.linux_generic._fat_mounts())
            _cliproc.assert_called_once_with('mount')
        self.assertEqual('/media/MBED_xxx', mount_dict['/dev/sdb'])
        self.assertEqual('/media/MBED___x', mount_dict['/dev/sdd'])
        self.assertEqual('/media/MBED-xxx', mount_dict['/dev/sde'])
        self.assertEqual('/media/MBED_x-x', mount_dict['/dev/sdc'])

        self.assertEqual('/mnt/NUCLEO', mount_dict['/dev/sda'])
        self.assertEqual('/mnt/NUCLEO_', mount_dict['/dev/sdf'])
        self.assertEqual('/mnt/DAPLINK', mount_dict['/dev/sdg'])
        self.assertEqual('/mnt/DAPLINK_', mount_dict['/dev/sdh'])
        self.assertEqual('/mnt/DAPLINK__', mount_dict['/dev/sdi'])

    def find_candidates_with_patch(self, mount_list, link_dict, listdir_dict):
        if not getattr(sys.modules['os'], 'readlink', None):
            sys.modules['os'].readlink = None

        with patch('mbed_lstools.linux.MbedLsToolsLinuxGeneric._run_cli_process') as _cliproc,\
             patch('os.readlink') as _readlink,\
             patch('os.listdir') as _listdir,\
             patch('mbed_lstools.linux.abspath') as _abspath,\
             patch('mbed_lstools.linux.isdir') as _isdir:
            _isdir.return_value = True
            _cliproc.return_value = (b'\n'.join(mount_list), None, 0)
            def do_readlink(link):
                # Fix for testing on Windows
                link = link.replace('\\', '/')
                return link_dict[link]
            _readlink.side_effect = do_readlink
            def do_listdir(dir):
                # Fix for testing on Windows
                dir = dir.replace('\\', '/')
                return listdir_dict[dir]
            _listdir.side_effect = do_listdir
            def do_abspath(dir):
                _, path = os.path.splitdrive(
                    os.path.normpath(os.path.join(os.getcwd(), dir)))
                path = path.replace('\\', '/')
                return path
            _abspath.side_effect = do_abspath
            ret_val = self.linux_generic.find_candidates()
            _cliproc.assert_called_once_with('mount')
            return ret_val


    listdir_dict_rpi = {
        '/dev/disk/by-id': [
            'usb-MBED_VFS_0240000028634e4500135006691700105f21000097969900-0:0',
            'usb-MBED_VFS_0240000028884e450018700f6bf000338021000097969900-0:0',
            'usb-MBED_VFS_0240000028884e45001f700f6bf000118021000097969900-0:0',
            'usb-MBED_VFS_0240000028884e450036700f6bf000118021000097969900-0:0',
            'usb-MBED_VFS_0240000029164e45001b0012706e000df301000097969900-0:0',
            'usb-MBED_VFS_0240000029164e45002f0012706e0006f301000097969900-0:0'
        ],
        '/dev/serial/by-id': [
            'usb-ARM_DAPLink_CMSIS-DAP_0240000028634e4500135006691700105f21000097969900-if01',
            'usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450018700f6bf000338021000097969900-if01',
            'usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450036700f6bf000118021000097969900-if01',
            'usb-ARM_DAPLink_CMSIS-DAP_0240000029164e45001b0012706e000df301000097969900-if01'
        ]
    }

    link_dict_rpi = {
        '/dev/disk/by-id/usb-MBED_VFS_0240000028634e4500135006691700105f21000097969900-0:0': '../../sdb',
        '/dev/disk/by-id/usb-MBED_VFS_0240000028884e450018700f6bf000338021000097969900-0:0': '../../sdc',
        '/dev/disk/by-id/usb-MBED_VFS_0240000028884e45001f700f6bf000118021000097969900-0:0': '../../sdd',
        '/dev/disk/by-id/usb-MBED_VFS_0240000028884e450036700f6bf000118021000097969900-0:0': '../../sde',
        '/dev/disk/by-id/usb-MBED_VFS_0240000029164e45001b0012706e000df301000097969900-0:0': '../../sdf',
        '/dev/disk/by-id/usb-MBED_VFS_0240000029164e45002f0012706e0006f301000097969900-0:0': '../../sdg',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000028634e4500135006691700105f21000097969900-if01': '../../ttyACM0',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450018700f6bf000338021000097969900-if01': '../../ttyACM1',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450036700f6bf000118021000097969900-if01': '../../ttyACM3',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000029164e45001b0012706e000df301000097969900-if01': '../../ttyACM2'
    }

    mount_list_rpi = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/usb2 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/usb3 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdf on /media/usb4 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdg on /media/usb5 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]
    def test_get_detected_rpi(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_rpi, self.link_dict_rpi, self.listdir_dict_rpi)

        self.assertIn({
            'mount_point': '/media/usb0',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': '0240000028634e4500135006691700105f21000097969900'
          }, mbed_det)
        self.assertIn({
            'mount_point': '/media/usb1',
            'serial_port': '/dev/ttyACM1',
            'target_id_usb_id': '0240000028884e450018700f6bf000338021000097969900'
          }, mbed_det)
        self.assertIn({
            'mount_point': '/media/usb4',
            'serial_port': '/dev/ttyACM2',
            'target_id_usb_id': '0240000029164e45001b0012706e000df301000097969900'
          }, mbed_det)

        self.assertIn({
            'mount_point': '/media/usb3',
            'serial_port': '/dev/ttyACM3',
            'target_id_usb_id': '0240000028884e450036700f6bf000118021000097969900'
          }, mbed_det)


    listdir_dict_1 = {
        '/dev/disk/by-id': [
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM',
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1',
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2',
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5',
            '/dev/disk/by-id/ata-TSSTcorpDVD-ROM_TS-H352C',
            '/dev/disk/by-id/usb-MBED_FDi_sk_A000000001-0:0',
            '/dev/disk/by-id/usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part1',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part2',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part5',
        ],
        '/dev/serial/by-id': [
            '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01',
            '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_A000000001-if01',
        ]
    }

    link_dict_1 = {
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM': '../../sda',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1': '../../sda1',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2': '../../sda2',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5': '../../sda5',
        '/dev/disk/by-id/ata-TSSTcorpDVD-ROM_TS-H352C': '../../sr0',
        '/dev/disk/by-id/usb-MBED_FDi_sk_A000000001-0:0': '../../sdc',
        '/dev/disk/by-id/usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0': '../../sdb',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77': '../../sda',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part1': '../../sda1',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part2': '../../sda2',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part5': '../../sda5',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01': '../../ttyACM1',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_A000000001-if01': '../../ttyACM0'
    }

    mount_list_1 = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]
    def test_get_detected_1_k64f(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_1, self.link_dict_1, self.listdir_dict_1)
        self.assertIn({
            'mount_point': '/media/usb0',
            'serial_port': '/dev/ttyACM1',
            'target_id_usb_id': '0240020152986E5EAF6693E6'
          }, mbed_det)

        self.assertIn({
            'mount_point': '/media/usb1',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': 'A000000001'
          }, mbed_det)


    listdir_dict_2 = {
        '/dev/disk/by-id': [
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM',
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1',
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2',
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5',
            'ata-TSSTcorpDVD-ROM_TS-H352C',
            'usb-MBED_FDi_sk_A000000001-0:0',
            'usb-MBED_microcontroller_02400201489A1E6CB564E3D4-0:0',
            'usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0',
            'usb-MBED_microcontroller_0240020152A06E54AF5E93EC-0:0',
            'usb-MBED_microcontroller_0672FF485649785087171742-0:0',
            'wwn-0x5000cca30ccffb77',
            'wwn-0x5000cca30ccffb77-part1',
            'wwn-0x5000cca30ccffb77-part2',
            'wwn-0x5000cca30ccffb77-part5'
        ],
        '/dev/serial/by-id': [
            'usb-MBED_MBED_CMSIS-DAP_02400201489A1E6CB564E3D4-if01',
            'usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01',
            'usb-MBED_MBED_CMSIS-DAP_0240020152A06E54AF5E93EC-if01',
            'usb-MBED_MBED_CMSIS-DAP_A000000001-if01',
            'usb-STMicroelectronics_STM32_STLink_0672FF485649785087171742-if02'
        ]
    }

    link_dict_2 = {
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM': '../../sda',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1': '../../sda1',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2': '../../sda2',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5': '../../sda5',
        '/dev/disk/by-id/ata-TSSTcorpDVD-ROM_TS-H352C': '../../sr0',
        '/dev/disk/by-id/usb-MBED_FDi_sk_A000000001-0:0': '../../sdc',
        '/dev/disk/by-id/usb-MBED_microcontroller_02400201489A1E6CB564E3D4-0:0': '../../sde',
        '/dev/disk/by-id/usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0': '../../sdb',
        '/dev/disk/by-id/usb-MBED_microcontroller_0240020152A06E54AF5E93EC-0:0': '../../sdf',
        '/dev/disk/by-id/usb-MBED_microcontroller_0672FF485649785087171742-0:0': '../../sdd',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77': '../../sda',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part1': '../../sda1',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part2': '../../sda2',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part5': '../../sda5',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_02400201489A1E6CB564E3D4-if01': '../../ttyACM3',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01': '../../ttyACM1',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152A06E54AF5E93EC-if01': '../../ttyACM4',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_A000000001-if01': '../../ttyACM0',
        '/dev/serial/by-id/usb-STMicroelectronics_STM32_STLink_0672FF485649785087171742-if02': '../../ttyACM2'
    }

    mount_list_2 = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/usb2 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/usb3 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdf on /media/usb4 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]
    def test_get_detected_2_k64f(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_2, self.link_dict_2, self.listdir_dict_2)

        self.assertIn({
            'mount_point': '/media/usb1',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': 'A000000001'
          },
          mbed_det)
        self.assertIn({
            'mount_point': '/media/usb2',
            'serial_port': '/dev/ttyACM2',
            'target_id_usb_id': '0672FF485649785087171742'
          },
          mbed_det)

        self.assertIn({
            'mount_point': '/media/usb4',
            'serial_port': '/dev/ttyACM4',
            'target_id_usb_id': '0240020152A06E54AF5E93EC'
          },
          mbed_det)

        self.assertIn({
            'mount_point': '/media/usb3',
            'serial_port': '/dev/ttyACM3',
            'target_id_usb_id': '02400201489A1E6CB564E3D4'
          },
          mbed_det)

        self.assertIn({
            'mount_point': '/media/usb0',
            'serial_port': '/dev/ttyACM1',
            'target_id_usb_id': '0240020152986E5EAF6693E6'
          },
          mbed_det)


    listdir_dict_4 = {
        '/dev/disk/by-id': [
            'ata-VMware_Virtual_SATA_CDRW_Drive_00000000000000000001',
            'ata-VMware_Virtual_SATA_CDRW_Drive_01000000000000000001',
            'usb-MBED_VFS_0240000033514e45001f500585d40014e981000097969900-0:0'
        ],
        '/dev/serial/by-id': [
            'pci-ARM_DAPLink_CMSIS-DAP_0240000033514e45001f500585d40014e981000097969900-if01'
        ]
    }

    link_dict_4 = {
        '/dev/disk/by-id/ata-VMware_Virtual_SATA_CDRW_Drive_00000000000000000001': '../../sr0',
        '/dev/disk/by-id/ata-VMware_Virtual_SATA_CDRW_Drive_01000000000000000001': '../../sr1',
        '/dev/disk/by-id/usb-MBED_VFS_0240000033514e45001f500585d40014e981000097969900-0:0': '../../sdb',
        '/dev/serial/by-id/pci-ARM_DAPLink_CMSIS-DAP_0240000033514e45001f500585d40014e981000097969900-if01': '../../ttyACM0'
    }

    mount_list_4 = [
        b'/dev/sdb on /media/przemek/DAPLINK type vfat (rw,nosuid,nodev,relatime,uid=1000,gid=1000,fmask=0022,dmask=0022,codepage=437,iocharset=iso8859-1,shortname=mixed,showexec,utf8,flush,errors=remount-ro,uhelper=udisks2)'
    ]
    def test_get_detected_3_k64f(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_4, self.link_dict_4, self.listdir_dict_4)

        self.assertIn({
            'mount_point': '/media/przemek/DAPLINK',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': '0240000033514e45001f500585d40014e981000097969900'
          },
          mbed_det)


if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
import subprocess
import plistlib
import platform

from .lstools_base import MbedLsToolsBase

import logging
logger = logging.getLogger("mbedls.lstools_darwin")
logger.addHandler(logging.NullHandler())
DEBUG = logging.DEBUG
del logging

mbed_volume_name_match = re.compile(r'\b(mbed|SEGGER MSD)\b', re.I)

def _find_TTY(obj):
    ''' Find the first tty (AKA IODialinDevice) that we can find in the
        children of the specified object, or None if no tty is present.
    '''
    try:
        return obj['IODialinDevice']
    except KeyError:
        for child in obj.get('IORegistryEntryChildren', []):
            found = _find_TTY(child)
            if found:
                return found
        return None


def _prune(current, keys):
    """ Reduce the amount of data we have to sift through to only 
        include the specified keys, and children that contain the
        specified keys
    """
    pruned_current = {k: current[k] for k in keys if k in current}
    pruned_children = list(
      filter(None, [_prune(c, keys) for c in 
                    current.get('IORegistryEntryChildren', [])]))
    keep_current = any(k in current for k in keys) or pruned_children
    if keep_current:
        if pruned_children:
            pruned_current['IORegistryEntryChildren'] = pruned_children
        return pruned_current
    else:
        return {}


def _dfs_usb_info(obj, parents):
    """ Find all of the usb info that we can from this particular IORegistry 
        tree with depth first search (and searching the parent stack....)
    """
    output = {}
    if  ('BSD Name' in obj and obj['BSD Name'].startswith('disk') and
         mbed_volume_name_match.search(obj['IORegistryEntryName'])):
        disk_id = obj['BSD Name']
        usb_info = {
                'serial':None,
             'vendor_id':None,
            'product_id':None,
                   'tty':None,
        }
        for parent in [obj] + parents:
            if 'USB Serial Number' in parent:
                usb_info['serial'] = parent['USB Serial Number']
            if 'idVendor' in parent and 'idProduct' in parent:
                usb_info['vendor_id'] = parent['idVendor']
                usb_info['product_id'] = parent['idProduct']
            if usb_info['serial']:
            	usb_info['tty'] = _find_TTY(parent)
            if all(usb_info.values()):
                break
        logger.debug("found usb info %r", usb_info)
        output[disk_id] = usb_info
    for child in obj.get('IORegistryEntryChildren', []):
        output.update(_dfs_usb_info(child, [obj] + parents))
    return output


class MbedLsToolsDarwin(MbedLsToolsBase):
    """ mbed-enabled platform detection on Mac OS X
    """

    def __init__(self, **kwargs):
        MbedLsToolsBase.__init__(self, **kwargs)
        self.mac_version = float('.'.join(platform.mac_ver()[0].split('.')[:2]))

    def find_candidates(self):
        # {volume_id: {serial:, vendor_id:, product_id:, tty:}}
        volumes = self._volumes()

        # {volume_id: mount_point}
        mounts = self._mount_points()
        return [
            {
                'mount_point': mounts[v],
                'serial_port': volumes[v]['tty'],
                'target_id_usb_id': volumes[v].get('serial')
            } for v in set(volumes.keys()) and set(mounts.keys())
            if v in mounts and v in volumes
        ]

    def _mount_points(self):
        ''' Returns map {volume_id: mount_point} '''
        diskutil_ls = subprocess.Popen(['diskutil', 'list', '-plist'], stdout=subprocess.PIPE)
        disks = plistlib.readPlist(diskutil_ls.stdout)
        diskutil_ls.wait()

        if logger.isEnabledFor(DEBUG):
            import pprint
            logger.debug("disks dict \n%s", pprint.PrettyPrinter(indent=2).pformat(disks))
        return {disk['DeviceIdentifier']: disk.get('MountPoint', None)
                for disk in disks['AllDisksAndPartitions']}

    def _volumes(self):
        ''' returns a map {volume_id: {serial:, vendor_id:, product_id:, tty:}'''

        # to find all the possible mbed volumes, we look for registry entries
        # under all possible USB tree which have a "BSD Name" that starts with
        # "disk" # (i.e. this is a USB disk), and have a IORegistryEntryName that
        # matches /\cmbed/
        # Once we've found a disk, we can search up for a parent with a valid
        # serial number, and then search down again to find a tty that's part
        # of the same composite device
        # ioreg -a -r -n <usb_controller_name> -l
        usb_controllers = ['AppleUSBXHCI', 'AppleUSBUHCI', 'AppleUSBEHCI',
                           'AppleUSBOHCI', 'IOUSBHostDevice']

        cmp_par = '-n'
        # For El Captain we need to list all the instances of (-c) rather than
        # compare names (-n)
        if self.mac_version >= 10.11:
            cmp_par = '-c'

        for usb_controller in usb_controllers:
            ioreg_usb = subprocess.Popen(['ioreg', '-a', '-r', cmp_par, usb_controller, '-l'], stdout=subprocess.PIPE)
            try:
                usb_tree = plistlib.readPlist(ioreg_usb.stdout)
            except:
                usb_tree = []
            ioreg_usb.wait()

        r = {}

        for name, obj in enumerate(usb_tree):
            pruned_obj = _prune(obj, ['USB Serial Number', 'idVendor', 'BSD Name',
                                      'IORegistryEntryName', 'idProduct', 'IODialinDevice'])
            if logger.isEnabledFor(DEBUG):
                import pprint
                logger.debug("finding in \n%s", pprint.PrettyPrinter(indent=2).pformat(pruned_obj))
            r.update(_dfs_usb_info(pruned_obj, []))

        logger.debug("_volumes return %r", r)
        return r



#!/usr/bin/env python

"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import sys
import json
import argparse
import platform
from collections import defaultdict

# Make sure that any global generic setup is run
from . import lstools_base

import logging
logger = logging.getLogger("mbedls.main")
logger.addHandler(logging.NullHandler())
del logging


def create(**kwargs):
    """! Factory used to create host OS specific mbed-lstools object

    :param kwargs: keyword arguments to pass along to the constructors
    @return Returns MbedLsTools object or None if host OS is not supported

    """
    result = None
    mbed_os = mbed_os_support()
    if mbed_os is not None:
        if mbed_os == 'Windows7':
            from .windows import MbedLsToolsWin7
            result = MbedLsToolsWin7(**kwargs)
        elif mbed_os == 'LinuxGeneric':
            from .linux import MbedLsToolsLinuxGeneric
            result = MbedLsToolsLinuxGeneric(**kwargs)
        elif mbed_os == 'Darwin':
            from .darwin import MbedLsToolsDarwin
            result = MbedLsToolsDarwin(**kwargs)
    return result


def mbed_os_support():
    """! Function used to determine if host OS is supported by mbed-lstools

    @return Returns None if host OS is not supported else return OS short name

    @details This function should be ported for new OS support
    """
    result = None
    os_info = mbed_lstools_os_info()
    if (os_info[0] == 'nt' and os_info[1] == 'Windows'):
        result = 'Windows7'
    elif (os_info[0] == 'posix' and os_info[1] == 'Linux'):
        result = 'LinuxGeneric'
    elif (os_info[0] == 'posix' and os_info[1] == 'Darwin'):
        result = 'Darwin'
    return result


def mbed_lstools_os_info():
    """! Returns information about host OS

    @return Returns tuple with information about OS and host platform
    """
    result = (os.name,
              platform.system(),
              platform.release(),
              platform.version(),
              sys.platform)
    return result


def get_version():
    """! Get mbed-ls Python module version string """
    import pkg_resources  # part of setuptools
    return pkg_resources.require("mbed-ls")[0].version

def print_version(mbeds, args):
    print(get_version())

def print_mbeds(mbeds, args, simple):
    devices = mbeds.list_mbeds(unique_names=True, read_details_txt=True)
    if devices:
        from prettytable import PrettyTable
        columns = ['platform_name', 'platform_name_unique', 'mount_point',
                    'serial_port', 'target_id', 'daplink_version']
        pt = PrettyTable(columns)
        pt.align = 'l'
        for d in devices:
            pt.add_row([d.get(col, None) or 'unknown' for col in columns])
        print(pt.get_string(border=not simple, header=not simple,
                            padding_width=1, sortby='platform_name_unique'))

def print_table(mbeds, args):
    return print_mbeds(mbeds, args, False)

def print_simple(mbeds, args):
    return print_mbeds(mbeds, args, True)

def mock_platform(mbeds, args):
    for token in args.mock.split(','):
        if ':' in token:
            oper = '+' # Default
            mid, platform_name = token.split(':')
            if mid and mid[0] in ['+', '-']:
                oper = mid[0]   # Operation (character)
                mid = mid[1:]   # We remove operation character
            mbeds.mock_manufacture_id(mid, platform_name, oper=oper)
        elif token and token[0] in ['-', '!']:
            # Operations where do not specify data after colon: --mock=-1234,-7678
            oper = token[0]
            mid = token[1:]
            mbeds.mock_manufacture_id(mid, 'dummy', oper=oper)
        else:
            logger.error("Could not parse mock from token: '%s'", token)

def list_platforms(mbeds, args):
    print(mbeds.list_manufacture_ids())

def mbeds_as_json(mbeds, args):
    print(json.dumps(mbeds.list_mbeds(unique_names=True,
                                      read_details_txt=True),
                     indent=4, sort_keys=True))

def json_by_target_id(mbeds, args):
    print(json.dumps({m['target_id']: m for m
                      in mbeds.list_mbeds(unique_names=True,
                                          read_details_txt=True)},
                     indent=4, sort_keys=True))

def json_platforms(mbeds, args):
    platforms = set()
    for d in mbeds.list_mbeds():
        platforms |= set([d['platform_name']])
    print(json.dumps(list(platforms), indent=4, sort_keys=True))

def json_platforms_ext(mbeds, args):
    platforms = defaultdict(lambda: 0)
    for d in mbeds.list_mbeds():
        platforms[d['platform_name']] += 1
    print(json.dumps(platforms, indent=4, sort_keys=True))

def parse_cli(to_parse):
    """! Parse the command line

    @return Retrun a namespace that contains:
     * command - python function to run
     * skip_retarget - bool indicting to skip retargeting
     * list_unmounted - list boards that are not mounted
     * debug - turn on debug logging
    """
    parser = argparse.ArgumentParser()
    parser.set_defaults(command=print_table)

    commands = parser.add_argument_group('sub commands')\
                     .add_mutually_exclusive_group()
    commands.add_argument(
        '-s', '--simple', dest='command', action='store_const',
        const=print_simple,
        help='list attached targets without column headers and borders')
    commands.add_argument(
        '-j', '--json', dest='command', action='store_const',
        const=mbeds_as_json,
        help='list attached targets with detailed information in JSON format')
    commands.add_argument(
        '-J', '--json-by-target-id', dest='command', action='store_const',
        const=json_by_target_id,
        help='map attached targets from their target ID to their detailed '
        'information in JSON format')
    commands.add_argument(
        '-p', '--json-platforms', dest='command', action='store_const',
        const=json_platforms,
        help='list attached platform names in JSON format.')
    commands.add_argument(
        '-P', '--json-platforms-ext', dest='command', action='store_const',
        const=json_platforms_ext,
        help='map attached platform names to the number of attached boards in '
        'JSON format')
    commands.add_argument(
        '-l', '--list', dest='command', action='store_const',
        const=list_platforms,
        help='list all target IDs and their corresponding platform names '
        'understood by mbed-ls')
    commands.add_argument(
        '--version', dest='command', action='store_const', const=print_version,
        help='print package version and exit')
    commands.add_argument(
        '-m', '--mock', metavar='ID:NAME',
        help='substitute or create a target ID to platform name mapping used'
        'when invoking mbedls in the current directory')

    parser.add_argument(
        '--skip-retarget', dest='skip_retarget', default=False,
        action="store_true",
        help='skip parsing and interpretation of the re-target file,'
        ' `./mbedls.json`')
    parser.add_argument(
        '-u', '--list-unmounted', dest='list_unmounted', default=False,
        action='store_true',
        help='list mbeds, regardless of whether they are mounted or not')
    parser.add_argument(
        '-d', '--debug', dest='debug', default=False, action="store_true",
        help='outputs extra debug information useful when creating issues!')

    args = parser.parse_args(to_parse)
    if args.mock:
        args.command = mock_platform
    return args

def start_logging():
    try:
        import colorlog
        colorlog.basicConfig(
            format='%(log_color)s%(levelname)s%(reset)s:%(name)s:%(message)s')
    except ImportError:
        import logging
        logging.basicConfig()
        del logging

def mbedls_main():
    """! Function used to drive CLI (command line interface) application
    @return Function exits with success code
    """
    start_logging()

    args = parse_cli(sys.argv[1:])

    import logging
    root_logger = logging.getLogger("mbedls")
    if args.debug:
        root_logger.setLevel(logging.DEBUG)
    else:
        root_logger.setLevel(logging.INFO)
    del logging
    logger.debug("mbed-ls ver. %s", get_version())
    logger.debug("host: %s",  str(mbed_lstools_os_info()))

    mbeds = create(skip_retarget=args.skip_retarget,
                   list_unmounted=args.list_unmounted,
                   force_mock=args.command is mock_platform)

    if mbeds is None:
        logger.critical('This platform is not supported! Pull requests welcome at github.com/ARMmbed/mbed-ls')
        sys.exit(-1)

    ret_code = args.command(mbeds, args)
    if not ret_code:
        ret_code = 0

    logger.debug("Return code: %d", ret_code)

    sys.exit(ret_code)



#!/usr/bin/env python
"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import errno
import logging
import re
import json
from io import StringIO
from mock import patch, mock_open
from copy import deepcopy

from mbed_lstools.lstools_base import MbedLsToolsBase, FSInteraction

class DummyLsTools(MbedLsToolsBase):
    return_value = []
    def find_candidates(self):
        return self.return_value

try:
    basestring
except NameError:
    # Python 3
    basestring = str

class BasicTestCase(unittest.TestCase):
    """ Basic test cases checking trivial asserts
    """

    def setUp(self):
        self.base = DummyLsTools()

    def tearDown(self):
        pass

    def test_list_mbeds_valid_platform(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"},
                                  {'mount_point': None,
                                   'target_id_usb_id': '00000000000',
                                   'serial_port': 'not_valid'}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_lstools.lstools_base.PlatformDatabase.get") as _get,\
             patch('os.listdir') as _listdir:
            _mpr.return_value = True
            _read_htm.return_value = (u'0241BEEFDEAD', {})
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            _listdir.return_value = []
            to_check = self.base.list_mbeds()
            _read_htm.assert_called_once_with('dummy_mount_point')
            _get.assert_called_once_with('0241', device_type='daplink', verbose_data=True)
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['target_id'], "0241BEEFDEAD")
        self.assertEqual(to_check[0]['platform_name'], 'foo_target')

    def test_list_mbeds_invalid_tid(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"},
                                  {'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': None,
                                   'serial_port': 'not_valid'}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_lstools.lstools_base.PlatformDatabase.get") as _get,\
             patch('os.listdir') as _listdir:
            _mpr.return_value = True
            _read_htm.side_effect = [(u'0241BEEFDEAD', {}), (None, {})]
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            _listdir.return_value = []
            to_check = self.base.list_mbeds()
            _get.assert_called_once_with('0241', device_type='daplink', verbose_data=True)
        self.assertEqual(len(to_check), 2)
        self.assertEqual(to_check[0]['target_id'], "0241BEEFDEAD")
        self.assertEqual(to_check[0]['platform_name'], 'foo_target')
        self.assertEqual(to_check[1]['target_id'], None)
        self.assertEqual(to_check[1]['platform_name'], None)

    def test_list_mbeds_invalid_platform(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'not_in_target_db',
                                   'serial_port': "dummy_serial_port"}]
        for qos in [FSInteraction.BeforeFilter, FSInteraction.AfterFilter]:
            with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
                patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
                patch("mbed_lstools.lstools_base.PlatformDatabase.get") as _get,\
                patch('os.listdir') as _listdir:
                _mpr.return_value = True
                _read_htm.return_value = (u'not_in_target_db', {})
                _get.return_value = None
                _listdir.return_value = []
                to_check = self.base.list_mbeds()
                _read_htm.assert_called_once_with('dummy_mount_point')
                _get.assert_called_once_with('not_', device_type='daplink', verbose_data=True)
            self.assertEqual(len(to_check), 1)
            self.assertEqual(to_check[0]['target_id'], "not_in_target_db")
            self.assertEqual(to_check[0]['platform_name'], None)

    def test_list_mbeds_unmount_mid_read(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch('os.listdir') as _listdir:
            _mpr.return_value = True
            _listdir.side_effect = OSError
            to_check = self.base.list_mbeds()
        self.assertEqual(len(to_check), 0)

    def test_list_mbeds_unmount_mid_read_list_unmounted(self):
        self.base.list_unmounted = True
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch('os.listdir') as _listdir:
            _mpr.return_value = True
            _listdir.side_effect = OSError
            to_check = self.base.list_mbeds()
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['mount_point'], None)
        self.assertEqual(to_check[0]['device_type'], 'unknown')

    def test_list_manufacture_ids(self):
        table_str = self.base.list_manufacture_ids()
        self.assertTrue(isinstance(table_str, basestring))

    def test_mock_manufacture_ids_default_multiple(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)
            self.assertEqual(platform_name, self.base.plat_db.get(mid))

    def test_mock_manufacture_ids_minus(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)
            self.assertEqual(platform_name, self.base.plat_db.get(mid))

        # oper='-'
        mock_ids = self.base.mock_manufacture_id('0342', '', oper='-')
        self.assertEqual('TEST_PLATFORM_NAME_1', self.base.plat_db.get("0341"))
        self.assertEqual(None, self.base.plat_db.get("0342"))
        self.assertEqual('TEST_PLATFORM_NAME_3', self.base.plat_db.get("0343"))

    def test_mock_manufacture_ids_star(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)

            self.assertEqual(platform_name, self.base.plat_db.get(mid))

        # oper='-'
        self.base.mock_manufacture_id('*', '', oper='-')
        self.assertEqual(None, self.base.plat_db.get("0341"))
        self.assertEqual(None, self.base.plat_db.get("0342"))
        self.assertEqual(None, self.base.plat_db.get("0343"))

    def test_update_device_from_fs_mid_unmount(self):
        dummy_mount = 'dummy_mount'
        device = {
            'mount_point': dummy_mount
        }

        with patch('os.listdir') as _listdir:
            _listdir.side_effect = OSError
            self.base._update_device_from_fs(device, False)
            self.assertEqual(device['device_type'], 'unknown')
            self.assertEqual(device['mount_point'], None)

    def test_update_device_from_fs_unknown(self):
        device = {}
        self.base._update_device_from_fs(device, False)
        self.assertEqual(device['device_type'], 'unknown')

    def test_detect_device_test(self):
        device_type = self.base._detect_device_type(['Segger.html'])
        self.assertEqual(device_type, 'jlink')

        device_type = self.base._detect_device_type(['MBED.HTM', 'DETAILS.TXT'])
        self.assertEqual(device_type, 'daplink')

    def test_update_device_details_jlink(self):
        jlink_html_contents = ('<html><head><meta http-equiv="refresh" '
                               'content="0; url=http://www.nxp.com/FRDM-KL27Z"/>'
                               '<title>NXP Product Page</title></head><body></body></html>')
        _open = mock_open(read_data=jlink_html_contents)
        dummy_mount_point = 'dummy'
        base_device = {
            'mount_point': dummy_mount_point
        }

        with patch('mbed_lstools.lstools_base.open', _open):
            device = deepcopy(base_device)
            self.base._update_device_details_jlink(device, False, ['Board.html', 'User Guide.html'])
            self.assertEqual(device['url'], 'http://www.nxp.com/FRDM-KL27Z')
            self.assertEqual(device['platform_name'], 'KL27Z')
            _open.assert_called_once_with(os.path.join(dummy_mount_point, 'Board.html'), 'r')

            _open.reset_mock()

            device = deepcopy(base_device)
            self.base._update_device_details_jlink(device, False, ['User Guide.html'])
            self.assertEqual(device['url'], 'http://www.nxp.com/FRDM-KL27Z')
            self.assertEqual(device['platform_name'], 'KL27Z')
            _open.assert_called_once_with(os.path.join(dummy_mount_point, 'User Guide.html'), 'r')

            _open.reset_mock()

            device = deepcopy(base_device)
            self.base._update_device_details_jlink(device, False, ['unhelpful_file.html'])
            self.assertEqual(device, base_device)
            _open.assert_not_called()

    def test_fs_never(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._update_device_from_fs") as _up_fs:
            filter = None
            ret = self.base._fs_never(deepcopy(device), filter, False)
            ret_with_details = self.base._fs_never(deepcopy(device), filter, True)
            self.assertIsNotNone(ret)
            self.assertIsNotNone(ret_with_details)
            self.assertEqual(ret['target_id'], ret['target_id_usb_id'])
            self.assertEqual(ret, ret_with_details)
            _up_fs.assert_not_called()

            filter_in = lambda m: m['platform_name'] == 'K64F'
            ret = self.base._fs_never(deepcopy(device), filter_in, False)
            ret_with_details = self.base._fs_never(deepcopy(device), filter_in, True)
            self.assertIsNotNone(ret)
            self.assertIsNotNone(ret_with_details)
            self.assertEqual(ret, ret_with_details)
            _up_fs.assert_not_called()

            filter_out = lambda m: m['platform_name'] != 'K64F'
            ret = self.base._fs_never(deepcopy(device), filter_out, False)
            ret_with_details = self.base._fs_never(deepcopy(device), filter_out, True)
            self.assertIsNone(ret)
            self.assertEqual(ret, ret_with_details)
            _up_fs.assert_not_called()

    def test_fs_after(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase._details_txt") as _up_details,\
             patch('os.listdir') as _listdir:
            new_device_id = "00017531642046"
            _read_htm.return_value = (new_device_id, {})
            _listdir.return_value = []
            _up_details.return_value = {
                'automation_allowed': '0'
            }
            filter = None
            ret = self.base._fs_after_id_check(deepcopy(device), filter, False)
            _up_details.assert_not_called()
            ret_with_details = self.base._fs_after_id_check(deepcopy(device), filter, True)

            self.assertIsNotNone(ret)
            self.assertIsNotNone(ret_with_details)
            self.assertEqual(ret['target_id'], new_device_id)
            self.assertEqual(ret_with_details['daplink_automation_allowed'], '0')
            self.assertDictContainsSubset(ret, ret_with_details)
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_in = lambda m: m['target_id'] == device['target_id_usb_id']
            filter_details = lambda m: m.get('daplink_automation_allowed', None) == '0'

            ret = self.base._fs_after_id_check(deepcopy(device), filter_in, False)
            ret_with_details = self.base._fs_after_id_check(deepcopy(device),
                                                            filter_details,
                                                            True)

            self.assertIsNotNone(ret)
            self.assertIsNone(ret_with_details)
            self.assertEqual(ret['target_id'], new_device_id)
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_not_called()

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_out = lambda m: m['target_id'] == new_device_id

            ret = self.base._fs_after_id_check(deepcopy(device), filter_out, False)
            ret_with_details = self.base._fs_after_id_check(deepcopy(device),
                                                            filter_details,
                                                            True)

            self.assertIsNone(ret)
            self.assertIsNone(ret_with_details)
            _read_htm.assert_not_called()
            _up_details.assert_not_called()

    def test_fs_before(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase._details_txt") as _up_details,\
             patch('os.listdir') as _listdir:
            new_device_id = u'00017575430420'
            _read_htm.return_value = (new_device_id, {})
            _listdir.return_value = []
            _up_details.return_value = {
                'automation_allowed': '0'
            }

            filter = None
            ret = self.base._fs_before_id_check(deepcopy(device), filter, False)
            _up_details.assert_not_called()

            ret_with_details = self.base._fs_before_id_check(deepcopy(device), filter, True)
            self.assertIsNotNone(ret)
            self.assertIsNotNone(ret_with_details)
            self.assertEqual(ret['target_id'], new_device_id)
            self.assertEqual(ret_with_details['daplink_automation_allowed'], '0')
            self.assertDictContainsSubset(ret, ret_with_details)
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_in = lambda m: m['target_id'] == '00017575430420'
            filter_in_details = lambda m: m['daplink_automation_allowed'] == '0'
            ret = self.base._fs_before_id_check(deepcopy(device), filter_in, False)
            _up_details.assert_not_called()

            ret_with_details = self.base._fs_before_id_check(deepcopy(device),
                                                             filter_in_details,
                                                             True)
            self.assertIsNotNone(ret)
            self.assertIsNotNone(ret_with_details)
            self.assertEqual(ret['target_id'], new_device_id)
            self.assertEqual(ret_with_details['daplink_automation_allowed'], '0')
            self.assertDictContainsSubset(ret, ret_with_details)
            _read_htm.assert_called_with(device['mount_point'])
            _up_details.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()
            _up_details.reset_mock()

            filter_out = lambda m: m['target_id'] == '024075309420ABCE'
            filter_out_details = lambda m: m['daplink_automation_allowed'] == '1'
            ret = self.base._fs_before_id_check(deepcopy(device), filter_out, False)
            _up_details.assert_not_called()

            ret_with_details = self.base._fs_before_id_check(deepcopy(device),
                                                             filter_out_details,
                                                             True)
            self.assertIsNone(ret)
            self.assertIsNone(ret_with_details)
            _read_htm.assert_called_with(device['mount_point'])

class RetargetTestCase(unittest.TestCase):
    """ Test cases that makes use of retargetting
    """

    def setUp(self):
        retarget_data = {
            '0240DEADBEEF': {
                'serial_port' : 'valid'
            }
        }

        _open = mock_open(read_data=json.dumps(retarget_data))

        with patch('os.path.isfile') as _isfile,\
             patch('mbed_lstools.lstools_base.open', _open):
            self.base = DummyLsTools()
            _open.assert_called()

    def tearDown(self):
        pass

    def test_list_mbeds_valid_platform(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': None}]
        with patch('mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids') as _read_htm,\
             patch('mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready') as _mpr,\
             patch('mbed_lstools.lstools_base.PlatformDatabase.get') as _get,\
             patch('os.listdir') as _listdir:
            _mpr.return_value = True
            _read_htm.return_value = (u'0240DEADBEEF', {})
            _get.return_value = {
                'platform_name': 'foo_target'
            }
            _listdir.return_value = []
            to_check = self.base.list_mbeds()
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['serial_port'], 'valid')

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2017 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""Functions that manage a platform database"""

import datetime
import json
import re
from collections import OrderedDict, defaultdict
from copy import copy
from io import open
from os import makedirs
from os.path import join, dirname, getmtime
from appdirs import user_data_dir
from fasteners import InterProcessLock

try:
    unicode
except NameError:
    unicode = str


import logging
logger = logging.getLogger("mbedls.platform_database")
logger.addHandler(logging.NullHandler())
del logging

LOCAL_PLATFORM_DATABASE = join(user_data_dir("mbedls"), "platforms.json")
LOCAL_MOCKS_DATABASE = join(user_data_dir("mbedls"), "mock.json")

DEFAULT_PLATFORM_DB = {
    u'daplink': {
        u'0001': u'LPC2368',
        u'0002': u'LPC2368',
        u'0003': u'LPC2368',
        u'0004': u'LPC2368',
        u'0005': u'LPC2368',
        u'0006': u'LPC2368',
        u'0007': u'LPC2368',
        u'0100': u'LPC2368',
        u'0183': u'UBLOX_C027',
        u'0200': u'KL25Z',
        u'0201': u'KW41Z',
        u'0210': u'KL05Z',
        u'0214': u'HEXIWEAR',
        u'0217': u'K82F',
        u'0218': u'KL82Z',
        u'0220': u'KL46Z',
        u'0227': u'MIMXRT1050_EVK',
        u'0230': u'K20D50M',
        u'0231': u'K22F',
        u'0240': u'K64F',
        u'0245': u'K64F',
        u'0250': u'KW24D',
        u'0261': u'KL27Z',
        u'0262': u'KL43Z',
        u'0300': u'MTS_GAMBIT',
        u'0305': u'MTS_MDOT_F405RG',
        u'0310': u'MTS_DRAGONFLY_F411RE',
        u'0311': u'K66F',
        u'0315': u'MTS_MDOT_F411RE',
        u'0350': u'XDOT_L151CC',
        u'0400': u'MAXWSNENV',
        u'0405': u'MAX32600MBED',
        u'0406': u'MAX32620MBED',
        u'0407': u'MAX32620HSP',
        u'0408': u'MAX32625NEXPAQ',
        u'0409': u'MAX32630FTHR',
        u'0415': u'MAX32625MBED',
        u'0450': u'MTB_UBLOX_ODIN_W2',
        u'0451': u'MTB_MXCHIP_EMW3166',
        u'0452': u'MTB_LAIRD_BL600',
        u'0453': u'MTB_MTS_XDOT',
        u'0454': u'MTB_MTS_DRAGONFLY',
        u'0455': u'MTB_UBLOX_NINA_B1',
        u'0456': u'MTB_MURATA_ABZ',
        u'0457': u'MTB_RAK811',
        u'0458': u'MTB_ADV_WISE_1510',
        u'0459': u'MTB_ADV_WISE_1530',
        u'0460': u'MTB_ADV_WISE_1570',
        u'0461': u'MTB_LAIRD_BL652',
        u'0462': u'MTB_USI_WM_BN_BM_22',
        u'0500': u'SPANSION_PLACEHOLDER',
        u'0505': u'SPANSION_PLACEHOLDER',
        u'0510': u'SPANSION_PLACEHOLDER',
        u'0602': u'EV_COG_AD3029LZ',
        u'0603': u'EV_COG_AD4050LZ',
        u'0700': u'NUCLEO_F103RB',
        u'0705': u'NUCLEO_F302R8',
        u'0710': u'NUCLEO_L152RE',
        u'0715': u'NUCLEO_L053R8',
        u'0720': u'NUCLEO_F401RE',
        u'0725': u'NUCLEO_F030R8',
        u'0730': u'NUCLEO_F072RB',
        u'0735': u'NUCLEO_F334R8',
        u'0740': u'NUCLEO_F411RE',
        u'0742': u'NUCLEO_F413ZH',
        u'0743': u'DISCO_F413ZH',
        u'0744': u'NUCLEO_F410RB',
        u'0745': u'NUCLEO_F303RE',
        u'0746': u'DISCO_F303VC',
        u'0747': u'NUCLEO_F303ZE',
        u'0750': u'NUCLEO_F091RC',
        u'0755': u'NUCLEO_F070RB',
        u'0760': u'NUCLEO_L073RZ',
        u'0764': u'DISCO_L475VG_IOT01A',
        u'0765': u'NUCLEO_L476RG',
        u'0766': u'SILICA_SENSOR_NODE',
        u'0770': u'NUCLEO_L432KC',
        u'0775': u'NUCLEO_F303K8',
        u'0777': u'NUCLEO_F446RE',
        u'0778': u'NUCLEO_F446ZE',
        u'0779': u'NUCLEO_L433RC_P',
        u'0780': u'NUCLEO_L011K4',
        u'0785': u'NUCLEO_F042K6',
        u'0788': u'DISCO_F469NI',
        u'0790': u'NUCLEO_L031K6',
        u'0791': u'NUCLEO_F031K6',
        u'0795': u'DISCO_F429ZI',
        u'0796': u'NUCLEO_F429ZI',
        u'0797': u'NUCLEO_F439ZI',
        u'0799': u'ST_PLACEHOLDER',
        u'0805': u'DISCO_L053C8',
        u'0810': u'DISCO_F334C8',
        u'0812': u'NUCLEO_F722ZE',
        u'0813': u'NUCLEO_H743ZI',
        u'0815': u'DISCO_F746NG',
        u'0816': u'NUCLEO_F746ZG',
        u'0817': u'DISCO_F769NI',
        u'0818': u'NUCLEO_F767ZI',
        u'0819': u'NUCLEO_F756ZG',
        u'0820': u'DISCO_L476VG',
        u'0821': u'NUCLEO_L452RE',
        u'0822': u'DISCO_L496AG',
        u'0823': u'NUCLEO_L496ZG',
        u'0824': u'LPC824',
        u'0826': u'NUCLEO_F412ZG',
        u'0827': u'NUCLEO_L486RG',
        u'0828': u'NUCLEO_L496ZG_P',
        u'0829': u'NUCLEO_L452RE_P',
        u'0830': u'DISCO_F407VG',
        u'0833': u'DISCO_L072CZ_LRWAN1',
        u'0835': u'NUCLEO_F207ZG',
        u'0839': u'NUCLEO_WB55RG',
        u'0840': u'B96B_F446VE',
        u'0900': u'XPRO_SAMR21',
        u'0905': u'XPRO_SAMW25',
        u'0910': u'XPRO_SAML21',
        u'0915': u'XPRO_SAMD21',
        u'1000': u'LPC2368',
        u'1001': u'LPC2368',
        u'1010': u'LPC1768',
        u'1017': u'HRM1017',
        u'1018': u'SSCI824',
        u'1019': u'TY51822R3',
        u'1022': u'RO359B',
        u'1034': u'LPC11U34',
        u'1040': u'LPC11U24',
        u'1045': u'LPC11U24',
        u'1050': u'LPC812',
        u'1054': u'LPC54114',
        u'1056': u'LPC546XX',
        u'1060': u'LPC4088',
        u'1061': u'LPC11U35_401',
        u'1062': u'LPC4088_DM',
        u'1070': u'NRF51822',
        u'1075': u'NRF51822_OTA',
        u'1080': u'OC_MBUINO',
        u'1090': u'RBLAB_NRF51822',
        u'1095': u'RBLAB_BLENANO',
        u'1100': u'NRF51_DK',
        u'1101': u'NRF52_DK',
        u'1102': u'NRF52840_DK',
        u'1105': u'NRF51_DK_OTA',
        u'1114': u'LPC1114',
        u'1120': u'NRF51_DONGLE',
        u'1130': u'NRF51822_SBK',
        u'1140': u'WALLBOT_BLE',
        u'1168': u'LPC11U68',
        u'1200': u'NCS36510',
        u'1234': u'UBLOX_C027',
        u'1235': u'UBLOX_C027',
        u'1236': u'UBLOX_EVK_ODIN_W2',
        u'1237': u'UBLOX_EVK_NINA_B1',
        u'1300': u'NUC472-NUTINY',
        u'1301': u'NUMBED',
        u'1302': u'NUMAKER_PFM_NUC472',
        u'1303': u'NUMAKER_PFM_M453',
        u'1304': u'NUMAKER_PFM_M487',
        u'1305': u'NUMAKER_PFM_M2351',
        u'1306': u'NUMAKER_PFM_NANO130',
        u'1307': u'NUMAKER_PFM_NUC240',
        u'1549': u'LPC1549',
        u'1600': u'LPC4330_M4',
        u'1605': u'LPC4330_M4',
        u'2000': u'EFM32_G8XX_STK',
        u'2005': u'EFM32HG_STK3400',
        u'2010': u'EFM32WG_STK3800',
        u'2015': u'EFM32GG_STK3700',
        u'2020': u'EFM32LG_STK3600',
        u'2025': u'EFM32TG_STK3300',
        u'2030': u'EFM32ZG_STK3200',
        u'2035': u'EFM32PG_STK3401',
        u'2040': u'EFM32PG12_STK3402',
        u'2041': u'TB_SENSE_12',
        u'2045': u'TB_SENSE_1',
        u'2100': u'XBED_LPC1768',
        u'2201': u'WIZWIKI_W7500',
        u'2202': u'WIZWIKI_W7500ECO',
        u'2203': u'WIZWIKI_W7500P',
        u'2500': u'ADV_WISE_1570',
        u'3001': u'LPC11U24',
        u'4000': u'LPC11U35_Y5_MBUG',
        u'4005': u'NRF51822_Y5_MBUG',
        u'4100': u'MOTE_L152RC',
        u'4337': u'LPC4337',
        u'4500': u'DELTA_DFCM_NNN40',
        u'4501': u'DELTA_DFBM_NQ620',
        u'4502': u'DELTA_DFCM_NNN50',
        u'4600': u'REALTEK_RTL8195AM',
        u'5000': u'ARM_MPS2',
        u'5001': u'ARM_MPS2_M0',
        u'5002': u'ARM_BEETLE_SOC',
        u'5003': u'ARM_MPS2_M0P',
        u'5005': u'ARM_MPS2_M0DS',
        u'5007': u'ARM_MPS2_M1',
        u'5009': u'ARM_MPS2_M3',
        u'5011': u'ARM_MPS2_M4',
        u'5015': u'ARM_MPS2_M7',
        u'5020': u'HOME_GATEWAY_6LOWPAN',
        u'5500': u'RZ_A1H',
        u'5501': u'GR_LYCHEE',
        u'6660': u'NZ32_SC151',
        u'7010': u'BLUENINJA_CDP_TZ01B',
        u'7011': u'TMPM066',
        u'7013': u'TMPM46B',
        u'7402': u'MBED_BR_HAT',
        u'7778': u'TEENSY3_1',
        u'8001': u'UNO_91H',
        u'8002': u'UNO_81C',
        u'8003': u'UNO_81AM',
        u'8004': u'UNO_81A',
        u'8080': u'FF1705_L151CC',
        u'8081': u'FF_LPC546XX',
        u'9001': u'LPC1347',
        u'9002': u'LPC11U24',
        u'9003': u'LPC1347',
        u'9004': u'ARCH_PRO',
        u'9006': u'LPC11U24',
        u'9007': u'LPC11U35_501',
        u'9008': u'XADOW_M0',
        u'9009': u'ARCH_BLE',
        u'9010': u'ARCH_GPRS',
        u'9011': u'ARCH_MAX',
        u'9012': u'SEEED_TINY_BLE',
        u'9900': u'NRF51_MICROBIT',
        u'C002': u'VK_RZ_A1H',
        u'C005': u'MTM_MTCONNECT04S',
        u'C006': u'VBLUNO51',
        u'C008': u'SAKURAIO_EVB_01',
        u'C030': u'UBLOX_C030_U201',
        u'C031': u'UBLOX_C030_N211',
        u'C032': u'UBLOX_C030_R404M',
        u'C033': u'UBLOX_C030_R410M',
        u'C034': u'UBLOX_C030_S200',
        u'C035': u'UBLOX_C030_R3121',
        u'FFFF': u'K20 BOOTLOADER',
        u'RIOT': u'RIOT',
    },
    u'jlink': {
        u'X349858SLYN': {
            u'platform_name': u'NRF52_DK',
            u'jlink_device_name': u'nRF52832_xxaa'
        },
        u'FRDM-KL25Z': {
            u'platform_name': u'KL25Z',
            u'jlink_device_name': u'MKL25Z128xxx4'
        },
        u'FRDM-KL27Z': {
            u'platform_name': u'KL27Z',
            u'jlink_device_name': u'MKL27Z64xxx4'
        },
        u'FRDM-KL43Z': {
            u'platform_name': u'KL43Z',
            u'jlink_device_name': u'MKL43Z256xxx4'
        }
    }
}


def _get_modified_time(path):
    try:
        mtime = getmtime(path)
    except OSError:
        mtime = 0
    return datetime.datetime.fromtimestamp(mtime)


def _older_than_me(path):
    return _get_modified_time(path) < _get_modified_time(__file__)


def _modify_data_format(data, verbose_data, simple_data_key='platform_name'):
    if isinstance(data, dict):
        if verbose_data:
            return data

        return data[simple_data_key]
    else:
        if verbose_data:
            return {
                simple_data_key: data
            }

        return data


def _overwrite_or_open(db):
    try:
        if db is LOCAL_PLATFORM_DATABASE and _older_than_me(db):
            raise ValueError("Platform Database is out of date")
        with open(db, encoding="utf-8") as db_in:
            return json.load(db_in)
    except (IOError, ValueError) as exc:
        if db is LOCAL_PLATFORM_DATABASE:
            logger.warning(
                "Error loading database %s: %s; Recreating", db, str(exc))
            try:
                makedirs(dirname(db))
            except OSError:
                pass
            try:
                with open(db, "w", encoding="utf-8") as out:
                    out.write(unicode(json.dumps(DEFAULT_PLATFORM_DB)))
            except IOError:
                pass
            return copy(DEFAULT_PLATFORM_DB)
        else:
            return {}


class PlatformDatabase(object):
    """Represents a union of multiple platform database files.
    Handles inter-process synchronization of database files.
    """

    target_id_pattern = re.compile(r'^[a-fA-F0-9]{4}$')

    def __init__(self, database_files, primary_database=None):
        """Construct a PlatformDatabase object from a series of platform database files"""
        self._prim_db = primary_database
        if not self._prim_db and len(database_files) == 1:
            self._prim_db = database_files[0]
        self._dbs = OrderedDict()
        self._keys = defaultdict(set)
        for db in database_files:
            new_db = _overwrite_or_open(db)
            first_value = None
            if new_db.values():
                first_value = next(iter(new_db.values()))
                if not isinstance(first_value, dict):
                    new_db = {
                        'daplink': new_db
                    }

            if new_db:
                for device_type in new_db:
                    duplicates = self._keys[device_type].intersection(set(new_db[device_type].keys()))
                    duplicates = set(["%s.%s" % (device_type, k) for k in duplicates])
                    if duplicates:
                        logger.warning(
                            "Duplicate platform ids found: %s,"
                            " ignoring the definitions from %s",
                            " ".join(duplicates), db)
                    self._dbs[db] = new_db
                    self._keys[device_type] = self._keys[device_type].union(new_db[device_type].keys())
            else:
                self._dbs[db] = new_db


    def items(self, device_type='daplink'):
        for db in self._dbs.values():
            for entry in db.get(device_type, {}).items():
                yield entry

    def all_ids(self, device_type='daplink'):
        return iter(self._keys[device_type])

    def get(self, index, default=None, device_type='daplink', verbose_data=False):
        """Standard lookup function. Works exactly like a dict. If 'verbose_data'
        is True, all data for the platform is returned as a dict."""
        for db in self._dbs.values():
            if device_type in db:
                maybe_answer = db[device_type].get(index, None)
                if maybe_answer:
                    return _modify_data_format(maybe_answer, verbose_data)

        return default

    def _update_db(self):
        if self._prim_db:
            lock = InterProcessLock("%s.lock" % self._prim_db)
            acquired = lock.acquire(blocking=False)
            if not acquired:
                logger.debug("Waiting 60 seconds for file lock")
                acquired = lock.acquire(blocking=True, timeout=60)
            if acquired:
                try:
                    with open(self._prim_db, "w", encoding="utf-8") as out:
                        out.write(unicode(
                            json.dumps(self._dbs[self._prim_db])))
                    return True
                finally:
                    lock.release()
            else:
                logger.error("Could not update platform database: "
                             "Lock acquire failed after 60 seconds")
                return False
        else:
            logger.error("Can't update platform database: "
                         "destination database is ambiguous")
            return False

    def add(self, id, platform_name, permanent=False, device_type='daplink'):
        """Add a platform to this database, optionally updating an origin
        database
        """
        if self.target_id_pattern.match(id):
            if self._prim_db:
                if device_type not in self._dbs[self._prim_db]:
                    self._dbs[self._prim_db][device_type] = {}
                self._dbs[self._prim_db][device_type][id] = platform_name
            else:
                cur_db = next(iter(self._dbs.values()))
                if device_type not in cur_db:
                    cur_db[device_type] = {}
                cur_db[device_type][id] = platform_name
            self._keys[device_type].add(id)
            if permanent:
                self._update_db()
        else:
            raise ValueError("Invald target id: %s" % id)

    def remove(self, id, permanent=False, device_type='daplink', verbose_data=False):
        """Remove a platform from this database, optionally updating an origin
        database. If 'verbose_data' is True, all data for the platform is returned
        as a dict.
        """
        logger.debug("Trying remove of %s", id)
        if id is '*' and device_type in self._dbs[self._prim_db]:
            self._dbs[self._prim_db][device_type] = {}
        for db in self._dbs.values():
            if device_type in db and id in db[device_type]:
                logger.debug("Removing id...")
                removed = db[device_type][id]
                del db[device_type][id]
                self._keys[device_type].remove(id)
                if permanent:
                    self._update_db()

                return _modify_data_format(removed, verbose_data)



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
from os.path import join, isdir, dirname, abspath
import os

from .lstools_base import MbedLsToolsBase

import logging
logger = logging.getLogger("mbedls.lstools_linux")
logger.addHandler(logging.NullHandler())
del logging

def _readlink(link):
    content = os.readlink(link)
    if content.startswith(".."):
        return abspath(join(dirname(link), content))
    else:
        return content


class MbedLsToolsLinuxGeneric(MbedLsToolsBase):
    """ mbed-enabled platform for Linux with udev
    """
    def __init__(self, **kwargs):
        """! ctor
        """
        MbedLsToolsBase.__init__(self, **kwargs)
        self.nlp = re.compile(
            r'(pci|usb)-[0-9a-zA-Z_-]*_(?P<usbid>[0-9a-zA-Z]*)-.*$')
        self.mmp = re.compile(
            r'(?P<dev>(/[^/ ]*)+) on (?P<dir>(/[^/ ]*)+) ')

    def find_candidates(self):
        disk_ids = self._dev_by_id('disk')
        serial_ids = self._dev_by_id('serial')
        mount_ids = dict(self._fat_mounts())
        logger.debug("Mount mapping %r", mount_ids)

        return [
            {
                'mount_point' : mount_ids.get(disk_dev),
                'serial_port' : serial_ids.get(disk_uuid),
                'target_id_usb_id' : disk_uuid
            } for disk_uuid, disk_dev in disk_ids.items()
        ]

    def _dev_by_id(self, device_type):
        """! Get a dict, USBID -> device, for a device class
        @param device_type The type of devices to search. For exmaple, "serial"
          looks for all serial devices connected to this computer
        @return A dict: Device USBID -> device file in /dev
        """
        dir = join("/dev", device_type, "by-id")
        if isdir(dir):
            to_ret = dict(self._hex_ids([join(dir, f) for f in os.listdir(dir)]))
            logger.debug("Found %s devices by id %r", device_type, to_ret)
            return to_ret
        else:
            logger.error("Could not get %s devices by id. "
                         "This could be because your Linux distribution "
                         "does not use udev, or does not create /dev/%s/by-id "
                         "symlinks. Please submit an issue to github.com/"
                         "armmbed/mbed-ls.", device_type, device_type)
            return {}

    def _fat_mounts(self):
        """! Lists mounted devices with vfat file system (potential mbeds)
        @result Returns list of all mounted vfat devices
        @details Uses Linux shell command: 'mount'
        """
        _stdout, _, retval = self._run_cli_process('mount')
        if not retval:
            for line in _stdout.splitlines():
                if b'vfat' in line:
                    match = self.mmp.search(line.decode('utf-8'))
                    if match:
                        yield match.group("dev"), match.group("dir")

    def _hex_ids(self, dev_list):
        """! Build a USBID map for a device list
        @param disk_list List of disks in a system with USBID decoration
        @return Returns map USBID -> device file in /dev
        @details Uses regular expressions to get a USBID (TargeTIDs) a "by-id"
          symbolic link
        """
        logger.debug("Converting device list %r", dev_list)
        for dl in dev_list:
            match = self.nlp.search(dl)
            if match:
                yield match.group("usbid"), _readlink(dl)
